library(KMsurv)
library(survival)
library(tidyverse)


# R function for computing p and p_tau ------------------------------------

p_theo <- function(copula = "clayton", copula_param = 1, d = "exp", sh1 = 1, sh2 = 1, sc1 = 1, sc2 = 1, tau = 1) {
  if (d == "exp"){
    u = exp(-sc1*tau)
    v = exp(-sc2*tau)
    E0 <- function(x) {x^(sc1/sc2)}
  }
  else if (d == "wei"){
    u = exp(-sc1*(tau^sh1))
    v = exp(-sc2*(tau^sh2))
    E0 <- function(x){exp(-sc1 * (-log(x)/sc2)^(sh1/sh2))}
  }
  else if (d == "gam"){
    u = 1-pgamma(tau, shape = sh1, scale = 1/sc1)
    v = 1-pgamma(tau, shape = sh2, scale = 1/sc2)
    E0 <- function(x){1-pgamma(qgamma(1-x, shape = sh2, scale = 1/sc2), shape = sh1, scale = 1/sc1)}
  }
  else if (d == "ln"){
    u = 1-plnorm(tau, meanlog = sh1, sdlog = sc1)
    v = 1-plnorm(tau, meanlog = sh2, sdlog = sc2)
    E0 <- function(x){1-plnorm(qlnorm(1-x, meanlog = sh2, sdlog = sc2), meanlog = sh1, sdlog = sc1)}
  }
  else if (d == "burr"){
    u = 1-(1+tau^(-sh1))^(-sc1)
    v = 1-(1+tau^(-sh2))^(-sc2)
    E0 <- function(x){1-(1+((1-x)^(-1/sc2)-1)^(sh1/sh2))^(-sc1)}
  }
  
  if(copula == "clayton"){
    p_t = (u^(-copula_param)+v^(-copula_param)-1)^(-1/copula_param)/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = E^(-copula_param) + x^(-copula_param) - 1
      x^(-copula_param - 1) * B^(-1/copula_param - 1)
    }
  }
  else if (copula == "gumbel"){
    p_t = exp(-((-log(u))^(copula_param+1)+(-log(v))^(copula_param+1))^(1/(copula_param+1)))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = (-log(E))^(copula_param+1) + (-log(x))^(copula_param+1)
      exp(-(B^(1/(copula_param+1)))) * B^(-copula_param/(copula_param+1)) * ((-log(x))^copula_param)/x
    }
  }
  else if (copula == "frank"){
    p_t = -1/copula_param*log(1+((exp(-copula_param*u)-1)*(exp(-copula_param*v)-1))/(exp(-copula_param)-1))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = 1+(exp(-copula_param*E)-1)*(exp(-copula_param*x)-1)/(exp(-copula_param)-1)
      C = -copula_param*exp(-copula_param*x)*(exp(-copula_param*E)-1)/(exp(-copula_param)-1)
      -1/copula_param/B*C
    }
  }
  else if (copula == "fgm"){
    p_t = (u*v+copula_param*u*v*(1-u)*(1-v))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B=E
      B+copula_param*B*(1-B)*(1-2*x)
    }
  }
  else if (copula == "gb"){
    p_t = (u*v*exp(-copula_param*log(u)*log(v)))/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E*(1-copula_param*log(E))*(x^(-copula_param*log(E)))
    }
  }
  else if (copula == "indep"){
    p_t = u*v/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E
    }
  }
  p <- NULL
  p[1] <- integrate(H_function, 0, 1)$value
  p[2] <- integrate(H_function, v, 1)$value + p_t
  return(p)
}

# tongue data -------------------------------------------------------------


#tongue data
data(tongue)
dat <- data.frame(tongue) %>% 
  transform(type = factor(type, levels = c(1,2)))

#follow-up time
tau <- min(max(dat$time[which(dat$delta == 1 & dat$type == "1")]), max(dat$time[which(dat$delta== 1 & dat$type == "2")]))

#MLE for the exponential hazard rates
mle_1 <- sum(dat[dat$type == "1",]$delta)/sum(dat[dat$type == "1",]$time)
mle_2 <- sum(dat[dat$type == "2",]$delta)/sum(dat[dat$type == "2",]$time)

#KM
result_s <- survfit(Surv(time,delta)~type, data = dat)

#plot KM (Figure 4)
x <- seq(0, 400, length = 1000)

plot(result_s, conf.int=F, col = c("#0EC3C7", "#F8776E"), lwd = 3, yaxt="n", xaxt="n")
axis(2,cex.axis=1.5)
axis(1,cex.axis=1.5)
mtext("Survival probability", side=2, line=2.2, cex=2)
mtext("Time (weeks)", side=1, line=2.2, cex=2)
curve(1 - pexp(x, rate = mle_1), 
      col="blue", lwd=2, add=TRUE, yaxt="n")
curve(1 - pexp(x, rate = mle_2), 
      col="red", lwd=2, add=TRUE, yaxt="n")
legend("topright", legend = c(bquote(S[1] ~ ": Aneuploid Tumor"), bquote(S[2] ~ ": Diploid Tumor")), title="Tumor DNA profile", col = c("#0EC3C7", "#F8776E"),lty = "solid", cex = 2, lwd = 5)
lines(list(167, 167), list(0.0, 0.75), lty = 2, lwd = 3)


#computing p_tau for KM (Table 3)
dat_sort <- dat %>% 
  arrange(type, time)

delta1 = dat_sort$delta[which(dat$type == "1")]
delta2 = dat_sort$delta[which(dat$type == "2")]
n1 = length(delta1)
n2 = length(delta2)

result_1 <- survfit(Surv(time,delta)~1, data = dat[which(dat$type == "1"),])
time1 <- dat_sort$time[which(dat$type == "1")]
surv1 <- summary(result_1, times = c(time1))$surv

result_2 <- survfit(Surv(time,delta)~1, data = dat[which(dat$type == "2"),])
time2 <- dat_sort$time[which(dat$type == "2")]
surv2 <- summary(result_2, times = c(time2))$surv

S1.positive = surv1
S1.negative = c(1,surv1[-length(surv1)])
S2.positive = surv2
S2.negative = c(1,surv2[-length(surv2)])
S1.t = c(1,surv1)
S2.t = c(1,surv2)
n1.risk = n1 : 1
n2.risk = n2 : 1

time1_sort <- dat_sort$time[which(dat$type == "1")]
time2_sort <- dat_sort$time[which(dat$type == "2")]

p <- 0

for (i in 1 : length(time2_sort)) {
  t_2i = time2_sort[i]
  p=p+0.5*(S1.t[length(which(time1_sort<=t_2i))+1]
           +S1.t[length(which(time1_sort< t_2i))+1])*
    (S2.negative[i]-S2.positive[i])
}

#P(X_1>X_2)+1/2P(X_1=X_2)
tau <- min(max(dat$time[which(dat$delta == 1 & dat$type == "1")]), max(dat$time[which(dat$delta== 1 & dat$type == "2")]))
p_all <- p+1/2*summary(result_1, times = tau)$surv*summary(result_2, times = tau)$surv


#Jackknife SE for KM (Table 3)
del <- numeric(80)
N <- 80
for (j in 1:N) {
  dat_j <- dat[-j,]
  delta1_j <- NULL
  delta2_j <- NULL
  delta1_j = dat_j$delta[which(dat_j$type == "1")]
  delta2_j = dat_j$delta[which(dat_j$type == "2")]
  delta1_j = delta1_j[complete.cases(delta1_j)]
  delta2_j = delta2_j[complete.cases(delta2_j)]
  n1_j = sum(is.na(delta1_j)==FALSE)
  n2_j = sum(is.na(delta2_j)==FALSE)
  
  dat_sort_j <- dat_j %>% 
    arrange(type, time)
  
  result1_j <- survfit(Surv(time,delta)~1, data = dat_sort_j[which(dat_sort_j$type == "1"),])
  time1_j <- dat_sort_j$time[which(dat_sort_j$type == "1")]
  time1_j = time1_j[complete.cases(time1_j)]
  surv1_j <- summary(result1_j, times = c(time1_j))$surv
  
  result2_j <- survfit(Surv(time,delta)~1, data = dat_sort_j[which(dat_sort_j$type == "2"),])
  time2_j <- dat_sort_j$time[which(dat_sort_j$type == "2")]
  time2_j = time2_j[complete.cases(time2_j)]
  surv2_j <- summary(result2_j, times = c(time2_j))$surv
  
  S1.pos_j = surv1_j
  S1.neg_j = c(1,surv1_j[-length(surv1_j)])
  S2.pos_j = surv2_j
  S2.neg_j = c(1,surv2_j[-length(surv2_j)])
  S1.t_j = c(1,surv1_j)
  S2.t_j = c(1,surv2_j)
  n1.risk_j = n1_j : 1
  n2.risk_j = n2_j : 1
  
  p_j <- 0
  for (i in 1 : length(time2_j)) {
    t_2i = time2_j[i]
    p_j=p_j+0.5*(S1.t_j[length(which(time1_j<=t_2i))+1]
             +S1.t_j[length(which(time1_j< t_2i))+1])*
      (S2.neg_j[i]-S2.pos_j[i])
  }
  
  tau_j <- min(max(dat_j$time[which(dat_j$delta == 1 & dat_j$type == "1")]), max(dat_j$time[which(dat_j$delta== 1 & dat_j$type == "2")]))
  del[j] <- p_j+1/2*summary(result1_j, times = tau_j)$surv*summary(result2_j, times = tau_j)$surv
}

var_j <- (N-1)^2/N*var(del)
se_j <- sqrt(var_j)
p_val <- pnorm((p_all-0.5)/se_j, 0,1, lower.tail = FALSE)*2

#p_tau
p_all <- round(p_all, 3)
#jackknife se, p-value
se_j <- round(se_j, 3)
p_val <- round(p_val, 3)

#Theoretical p (Table 3)
#p and p_tau, SE for theoretical p
SE_data <- function(copula = "clayton", copula_parameter, tau) {
  N = 80
  del1 = numeric(N)
  del2 = numeric(N)
  #p for all data
  mle_1 = sum(dat[dat$type == "1",]$delta)/sum(dat[dat$type == "1",]$time)
  mle_2 = sum(dat[dat$type == "2",]$delta)/sum(dat[dat$type == "2",]$time)
  sh1 = 1
  sh2 = 1
  sc1 = mle_1
  sc2 = mle_2
  p_all = p_theo(copula = copula, copula_param = copula_parameter, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau)
  for (j in 1:N) {
    dat_j = dat[-j,]
    mle1_1 = sum(dat_j[dat_j$type == "1",]$delta)/sum(dat_j[dat_j$type == "1",]$time)
    mle2_1 = sum(dat_j[dat_j$type == "2",]$delta)/sum(dat_j[dat_j$type == "2",]$time)
    sh1_j = 1
    sh2_j = 1
    sc1_j = mle1_1
    sc2_j = mle2_1
    del = NULL
    del = p_theo(copula = copula, copula_param = copula_parameter, d = "exp", sh1 = sh1_j, sh2 = sh2_j, sc1 = sc1_j, sc2 = sc2_j, tau = tau)
    del1[j] = del[1]
    del2[j] = del[2]
  }
  var_j = (N-1)^2/N*var(del1)
  se_j = sqrt(var_j)
  p_val = pnorm((p_all[1]-0.5)/se_j, 0,1, lower.tail = FALSE)*2
  se_j = round(se_j, 3)
  p_val = round(p_val, 3)
  
  var_tau_j = (N-1)^2/N*var(del2)
  se_tau_j = sqrt(var_tau_j)
  p_val_tau = pnorm((p_all[2]-0.5)/se_tau_j, 0,1, lower.tail = FALSE)*2
  se_tau_j = round(se_tau_j, 3)
  p_val_tau = round(p_val_tau, 3)
  
  p_all = round(p_all, 3)
  
  result_se = matrix(c(p_all, se_j, p_val, se_tau_j, p_val_tau), ncol = 6)
  colnames(result_se) = c("p", "p_tau", "se_p", "p-value_1", "se_p_tau", "p-value_2")
  return(result_se)
}


#Indep
SE_data(copula = "indep" , copula_parameter = 1, tau = 167)

#Clayton
SE_data(copula = "clayton" , copula_parameter = 1, tau = 167)
SE_data(copula = "clayton" , copula_parameter = 5, tau = 167)

#Gumbel
SE_data(copula = "gumbel" , copula_parameter = 4, tau = 167)

#Frank
SE_data(copula = "frank" , copula_parameter = -20, tau = 167)
SE_data(copula = "frank" , copula_parameter = -5, tau = 167)
SE_data(copula = "frank" , copula_parameter = 5, tau = 167)

#FGM
SE_data(copula = "fgm" , copula_parameter = -1, tau = 167)
SE_data(copula = "fgm" , copula_parameter = 1, tau = 167)

#GB
SE_data(copula = "gb" , copula_parameter = 0.5, tau = 167)
SE_data(copula = "gb" , copula_parameter = 1, tau = 167)


