library(survival)
library(asaur)
library(tidyverse)

# R function for computing p and p_tau ------------------------------------

p_theo <- function(copula = "clayton", copula_param = 1, d = "exp", sh1 = 1, sh2 = 1, sc1 = 1, sc2 = 1, tau = 1) {
  if (d == "exp"){
    u = exp(-sc1*tau)
    v = exp(-sc2*tau)
    E0 <- function(x) {x^(sc1/sc2)}
  }
  else if (d == "wei"){
    u = exp(-sc1*(tau^sh1))
    v = exp(-sc2*(tau^sh2))
    E0 <- function(x){exp(-sc1 * (-log(x)/sc2)^(sh1/sh2))}
  }
  else if (d == "gam"){
    u = 1-pgamma(tau, shape = sh1, scale = 1/sc1)
    v = 1-pgamma(tau, shape = sh2, scale = 1/sc2)
    E0 <- function(x){1-pgamma(qgamma(1-x, shape = sh2, scale = 1/sc2), shape = sh1, scale = 1/sc1)}
  }
  else if (d == "ln"){
    u = 1-plnorm(tau, meanlog = sh1, sdlog = sc1)
    v = 1-plnorm(tau, meanlog = sh2, sdlog = sc2)
    E0 <- function(x){1-plnorm(qlnorm(1-x, meanlog = sh2, sdlog = sc2), meanlog = sh1, sdlog = sc1)}
  }
  else if (d == "burr"){
    u = 1-(1+tau^(-sh1))^(-sc1)
    v = 1-(1+tau^(-sh2))^(-sc2)
    E0 <- function(x){1-(1+((1-x)^(-1/sc2)-1)^(sh1/sh2))^(-sc1)}
  }
  
  if(copula == "clayton"){
    p_t = (u^(-copula_param)+v^(-copula_param)-1)^(-1/copula_param)/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = E^(-copula_param) + x^(-copula_param) - 1
      x^(-copula_param - 1) * B^(-1/copula_param - 1)
    }
  }
  else if (copula == "gumbel"){
    p_t = exp(-((-log(u))^(copula_param+1)+(-log(v))^(copula_param+1))^(1/(copula_param+1)))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = (-log(E))^(copula_param+1) + (-log(x))^(copula_param+1)
      exp(-(B^(1/(copula_param+1)))) * B^(-copula_param/(copula_param+1)) * ((-log(x))^copula_param)/x
    }
  }
  else if (copula == "frank"){
    p_t = -1/copula_param*log(1+((exp(-copula_param*u)-1)*(exp(-copula_param*v)-1))/(exp(-copula_param)-1))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = 1+(exp(-copula_param*E)-1)*(exp(-copula_param*x)-1)/(exp(-copula_param)-1)
      C = -copula_param*exp(-copula_param*x)*(exp(-copula_param*E)-1)/(exp(-copula_param)-1)
      -1/copula_param/B*C
    }
  }
  else if (copula == "fgm"){
    p_t = (u*v+copula_param*u*v*(1-u)*(1-v))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B=E
      B+copula_param*B*(1-B)*(1-2*x)
    }
  }
  else if (copula == "gb"){
    p_t = (u*v*exp(-copula_param*log(u)*log(v)))/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E*(1-copula_param*log(E))*(x^(-copula_param*log(E)))
    }
  }
  else if (copula == "indep"){
    p_t = u*v/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E
    }
  }
  p <- NULL
  p[1] <- integrate(H_function, 0, 1)$value
  p[2] <- integrate(H_function, v, 1)$value + p_t
  return(p)
}

# prostate data -----------------------------------------------------------

#prostate data
dat <- data.frame(prostateSurvival)

dat <- within(dat, {
  status.prost <- as.numeric({status == 1})
  status.other <- as.numeric({status == 2})})

#follow-up time
tau <- min(max(dat$survTime[which(dat$status.prost==1 & dat$grade == "mode")]), max(dat$survTime[which(dat$status.prost== 1 & dat$grade == "poor")]))

#MLE for the exponential hazard rates
mle_1 <- sum(dat[dat$grade == "mode",]$status.prost)/sum(dat[dat$grade == "mode",]$survTime)

mle_2 <- sum(dat[dat$grade == "poor",]$status.prost)/sum(dat[dat$grade == "poor",]$survTime)


#KM
result_s <- survfit(Surv(survTime, status.prost)~grade, data = dat)

#plot KM (Figure 6)
x <- seq(0, 400, length = 1000)
plot(result_s, conf.int=F, col = c("#0EC3C7", "#F8776E"), lwd = 3, yaxt="n", xaxt="n")
axis(2,cex.axis=1.5)
axis(1,cex.axis=1.5)
mtext("Survival probability", side=2, line=2.2, cex=2)
mtext("Time", side=1, line=2.2, cex=2)
curve(1 - pexp(x, rate = mle_1), 
      col="blue", lwd=2, add=TRUE, yaxt="n")
curve(1 - pexp(x, rate = mle_2), 
      col="red", lwd=2, add=TRUE, yaxt="n")
legend("bottomright", legend = c(bquote(S[1] ~ ": moderate"), bquote(S[2] ~ ": poor")), title="grade", col = c("#0EC3C7", "#F8776E"),lty = "solid", cex = 2, lwd = 5)
lines(list(108, 108), list(0.25, 1.2), lty = 2, lwd = 3)
abline(v = 108)

#calculate p_tau for KM(Table 4)
dat_sort <- dat %>% 
  arrange(grade, survTime, desc(status.prost))

delta1 = dat_sort$status.prost[which(dat_sort$grade == "mode")]
delta2 = dat_sort$status.prost[which(dat_sort$grade == "poor")]
delta1 = delta1[complete.cases(delta1)]
delta2 = delta2[complete.cases(delta2)]
n1 = length(delta1)
n2 = length(delta2)

result1 <- survfit(Surv(survTime, status.prost)~1, data = dat[which(dat$grade == "mode"),])
time1 <- dat_sort$survTime[which(dat_sort$grade == "mode")]
surv1 <- summary(result1, times = c(time1))$surv

result2 <- survfit(Surv(survTime, status.prost)~1, data = dat[which(dat$grade == "poor"),])
time2 <- dat_sort$survTime[which(dat_sort$grade == "poor")]
surv2 <- summary(result2, times = c(time2))$surv


S1.positive = surv1
S1.negative = c(1,surv1[-length(surv1)])
S2.positive = surv2
S2.negative = c(1,surv2[-length(surv2)])
S1.t = c(1,surv1)
S2.t = c(1,surv2)
n1.risk = n1 : 1
n2.risk = n2 : 1

p <- 0
for (i in 1 : length(time2)) {
  t_2i = time2[i]
  p = p + 0.5*(S1.t[length(which(time1<=t_2i))+1]
               +S1.t[length(which(time1< t_2i))+1])*
    (S2.negative[i]-S2.positive[i])
}

#P(X_1>X_2)+1/2P(X_1=X_2)
p_all <- p + 1/2*summary(result1, times = tau)$surv*summary(result2, times = tau)$surv


#Jackknife SE for KM (Table 4)
N <- 14294
del <- numeric(N)
for (j in 1:N) {
  dat_j <- dat[-j,]
  delta1_j = dat_j$status.prost[which(dat_j$grade == "mode")]
  delta2_j = dat_j$status.prost[which(dat_j$grade == "poor")]
  delta1_j = delta1_j[complete.cases(delta1_j)]
  delta2_j = delta2_j[complete.cases(delta2_j)]
  n1_j = length(delta1_j)
  n2_j = length(delta2_j)
  dat_sort <- dat_j %>% 
    arrange(grade, survTime, desc(status.prost))
  
  result1_j <- survfit(Surv(survTime, status.prost)~1, data = dat_sort[which(dat_sort$grade == "mode"),])
  time1_j <- dat_sort$survTime[which(dat_sort$grade == "mode")]
  time1_j = time1_j[complete.cases(time1_j)]
  surv1_j <- summary(result1_j, times = c(time1_j))$surv
  
  result2_j <- survfit(Surv(survTime, status.prost)~1, data = dat_sort[which(dat_sort$grade == "poor"),])
  time2_j <- dat_sort$survTime[which(dat_sort$grade == "poor")]
  time2_j = time2_j[complete.cases(time2_j)]
  surv2_j <- summary(result2_j, times = c(time2_j))$surv
  
  S1.pos_j = surv1_j
  S1.neg_j = c(1,surv1_j[-length(surv1_j)])
  S2.pos_j = surv2_j
  S2.neg_j = c(1,surv2_j[-length(surv2_j)])
  S1.t_j = c(1,surv1_j)
  S2.t_j = c(1,surv2_j)
  n1.risk_j = n1_j : 1
  n2.risk_j = n2_j : 1
  
  p_j = 0
  
  for (i in 1 : length(time2_j)) {
    t_2i = time2_j[i]
    p_j = p_j + 0.5*(S1.t_j[length(which(time1_j <= t_2i)) + 1]
                 + S1.t_j[length(which(time1_j < t_2i)) + 1])*
      (S2.neg_j[i] - S2.pos_j[i])
  }
  tau_j <- min(max(dat_j$survTime[which(dat_j$status.prost==1 & dat_j$grade == "mode")]), max(dat_j$survTime[which(dat_j$status.prost== 1 & dat_j$grade == "poor")]))
  del[j] <- p_j+1/2*summary(result1_j, times = tau_j)$surv*summary(result2_j, times = tau_j)$surv
}

#jackknife var, se, p-value
var_j <- (N-1)^2/N*var(del)
se_j <- sqrt(var_j)
p_val <- pnorm((p_all-0.5)/se_j, 0,1, lower.tail = FALSE)*2


#p_tau
p_all <- round(p_all, 3)
#jackknife se, p-value
se_j <- round(se_j, 3)
p_val <- round(p_val, 3)



#Theoretical p
#p, p_tau, and SE for theoretical p (Table 4)
SE_data <- function(copula = "clayton", copula_parameter, tau) {
  N <- nrow(dat)
  del1 <- numeric(N)
  del2 <- numeric(N)
  #p for all data
  mle_1 <- sum(dat[dat$grade == "mode",]$status.prost)/sum(dat[dat$grade == "mode",]$survTime)
  mle_2 <- sum(dat[dat$grade == "poor",]$status.prost)/sum(dat[dat$grade == "poor",]$survTime)
  sh1 <- 1
  sh2 <- 1
  sc1 <- mle_1
  sc2 <- mle_2
  p_all <- p_theo(copula = copula, copula_param = copula_parameter, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau)
  for (j in 1:N) {
    dat_j <- dat[-j,]
    mle1_1 <- sum(dat_j[dat_j$grade == "mode",]$status.prost)/sum(dat_j[dat_j$grade == "mode",]$survTime)
    mle2_1 <- sum(dat_j[dat_j$grade == "poor",]$status.prost)/sum(dat_j[dat_j$grade == "poor",]$survTime)
    sh1_j <- 1
    sh2_j <- 1
    sc1_j <- mle1_1
    sc2_j <- mle2_1
    del <- NULL
    del <- p_theo(copula = copula, copula_param = copula_parameter, d = "exp", sh1 = sh1_j, sh2 = sh2_j, sc1 = sc1_j, sc2 = sc2_j, tau = tau)
    del1[j] <- del[1]
    del2[j] <- del[2]
  }
  var_j <- (N-1)^2/N*var(del1)
  se_j <- sqrt(var_j)
  p_val <- pnorm((p_all[1]-0.5)/se_j, 0,1, lower.tail = FALSE)*2
  se_j <- round(se_j, 3)
  p_val <- round(p_val, 3)
  
  var_tau_j <- (N-1)^2/N*var(del2)
  se_tau_j <- sqrt(var_tau_j)
  p_val_tau <- pnorm((p_all[2]-0.5)/se_tau_j, 0,1, lower.tail = FALSE)*2
  se_tau_j <- round(se_tau_j, 3)
  p_val_tau <- round(p_val_tau, 3)
  
  p_all <- round(p_all, 3)
  
  result_se <- matrix(c(p_all, se_j, p_val, se_tau_j, p_val_tau), ncol = 6)
  colnames(result_se) <- c("p", "p_tau", "se_p", "p-value_1", "se_p_tau", "p-value_2")
  return(result_se)
}

#Indep
SE_data(copula = "indep" , copula_parameter = 1, tau = 108)

#Clayton
SE_data(copula = "clayton" , copula_parameter = 1, tau = 108)
SE_data(copula = "clayton" , copula_parameter = 5, tau = 108)

#Gumbel
SE_data(copula = "gumbel" , copula_parameter = 4, tau = 108)

#Frank
SE_data(copula = "frank" , copula_parameter = -20, tau = 108)
SE_data(copula = "frank" , copula_parameter = -5, tau = 108)
SE_data(copula = "frank" , copula_parameter = 5, tau = 108)

#FGM
SE_data(copula = "fgm" , copula_parameter = -1, tau = 108)
SE_data(copula = "fgm" , copula_parameter = 1, tau = 108)

#GB
SE_data(copula = "gb" , copula_parameter = 0.5, tau = 108)
SE_data(copula = "gb" , copula_parameter = 1, tau = 108)
