
library(tidyverse)
library(copula)
library(Copula.surv)


# parameters --------------------------------------------------------------

copula_param <- 1
sh1 <- 1
sh2 <- 1
sc1 <- 1
sc2 <- 2

# theoretical value ------------------------------------------------------------------

p_theo <- function(copula = "clayton", copula_param = 1, d = "exp", sh1 = 1, sh2 = 1, sc1 = 1, sc2 = 1, tau = 1) {
  if (d == "exp"){
    u = exp(-sc1*tau)
    v = exp(-sc2*tau)
    E0 <- function(x) {x^(sc1/sc2)}
  }
  else if (d == "wei"){
    u = exp(-sc1*(tau^sh1))
    v = exp(-sc2*(tau^sh2))
    E0 <- function(x){exp(-sc1 * (-log(x)/sc2)^(sh1/sh2))}
  }
  else if (d == "gam"){
    u = 1-pgamma(tau, shape = sh1, scale = 1/sc1)
    v = 1-pgamma(tau, shape = sh2, scale = 1/sc2)
    E0 <- function(x){1-pgamma(qgamma(1-x, shape = sh2, scale = 1/sc2), shape = sh1, scale = 1/sc1)}
  }
  else if (d == "ln"){
    u = 1-plnorm(tau, meanlog = sh1, sdlog = sc1)
    v = 1-plnorm(tau, meanlog = sh2, sdlog = sc2)
    E0 <- function(x){1-plnorm(qlnorm(1-x, meanlog = sh2, sdlog = sc2), meanlog = sh1, sdlog = sc1)}
  }
  else if (d == "burr"){
    u = 1-(1+tau^(-sh1))^(-sc1)
    v = 1-(1+tau^(-sh2))^(-sc2)
    E0 <- function(x){1-(1+((1-x)^(-1/sc2)-1)^(sh1/sh2))^(-sc1)}
  }
  
  if(copula == "clayton"){
    p_t = (u^(-copula_param)+v^(-copula_param)-1)^(-1/copula_param)/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = E^(-copula_param) + x^(-copula_param) - 1
      x^(-copula_param - 1) * B^(-1/copula_param - 1)
    }
  }
  else if (copula == "gumbel"){
    p_t = exp(-((-log(u))^(copula_param+1)+(-log(v))^(copula_param+1))^(1/(copula_param+1)))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = (-log(E))^(copula_param+1) + (-log(x))^(copula_param+1)
      exp(-(B^(1/(copula_param+1)))) * B^(-copula_param/(copula_param+1)) * ((-log(x))^copula_param)/x
    }
  }
  else if (copula == "frank"){
    p_t = -1/copula_param*log(1+((exp(-copula_param*u)-1)*(exp(-copula_param*v)-1))/(exp(-copula_param)-1))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B = 1+(exp(-copula_param*E)-1)*(exp(-copula_param*x)-1)/(exp(-copula_param)-1)
      C = -copula_param*exp(-copula_param*x)*(exp(-copula_param*E)-1)/(exp(-copula_param)-1)
      -1/copula_param/B*C
    }
  }
  else if (copula == "fgm"){
    p_t = (u*v+copula_param*u*v*(1-u)*(1-v))/2
    H_function <- function(x) {
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      B=E
      B+copula_param*B*(1-B)*(1-2*x)
    }
  }
  else if (copula == "gb"){
    p_t = (u*v*exp(-copula_param*log(u)*log(v)))/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E*(1-copula_param*log(E))*(x^(-copula_param*log(E)))
    }
  }
  else if (copula == "indep"){
    p_t = u*v/2
    H_function <- function(x){
      E = E0(x)
      RR = (E == rep(1, length(E)))
      E[RR] = 1-10^8
      E
    }
  }
  p <- matrix(c(copula, copula_param, tau, round(integrate(H_function, 0, 1)$value, 3), round(integrate(H_function, v, 1)$value + p_t, 3)), ncol = 5)
  #"p" is the theoretical value with tau = infinity, "p_tau" is with selected tau
  colnames(p) <- c("copula", "copula parameter", "tau", "p", "p_tau")
  return(p)
}

#simulation study (Table 2)

#follow-up time
c_tau <- c(0.5, 2, 5)

#exponential distribution
#parameter
sh1 <- 1
sh2 <- 1
sc1 <- 1
sc2 <- 2
#clayton
c_param <- c(1, 5, 10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "clayton", copula_param = copula_param, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gumbel", copula_param = copula_param, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "frank", copula_param = copula_param, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#fgm
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "fgm", copula_param = copula_param, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gb
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gb", copula_param = copula_param, d = "exp", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#Weibull distribution
#parameter
sh1 <- 0.5
sh2 <- 1
sc1 <- 1
sc2 <- 2
#clayton
c_param <- c(1, 5, 10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "clayton", copula_param = copula_param, d = "wei", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gumbel", copula_param = copula_param, d = "wei", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "frank", copula_param = copula_param, d = "wei", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#fgm
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "fgm", copula_param = copula_param, d = "wei", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gb
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gb", copula_param = copula_param, d = "wei", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gamma distribution
#parameter
sh1 <- 1.5
sh2 <- 2.0
sc1 <- 1
sc2 <- 2
#clayton
c_param <- c(1, 5, 10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "clayton", copula_param = copula_param, d = "gam", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gumbel", copula_param = copula_param, d = "gam", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "frank", copula_param = copula_param, d = "gam", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#fgm
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "fgm", copula_param = copula_param, d = "gam", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gb
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gb", copula_param = copula_param, d = "gam", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}


#log-normal distribution
#parameter
sh1 <- 0.7
sh2 <- 0.3
sc1 <- 1.5
sc2 <- 2.0
#clayton
c_param <- c(1, 5, 10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "clayton", copula_param = copula_param, d = "ln", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gumbel", copula_param = copula_param, d = "ln", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "frank", copula_param = copula_param, d = "ln", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#fgm
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "fgm", copula_param = copula_param, d = "ln", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gb
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gb", copula_param = copula_param, d = "ln", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}


#Burr III distribution
#parameter
sh1 <- 1.5
sh2 <- 1.0
sc1 <- 3.0
sc2 <- 1.0
#clayton
c_param <- c(1, 5, 10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "clayton", copula_param = copula_param, d = "burr", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gumbel", copula_param = copula_param, d = "burr", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "frank", copula_param = copula_param, d = "burr", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#fgm
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "fgm", copula_param = copula_param, d = "burr", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}

#gb
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(p_theo(copula = "gb", copula_param = copula_param, d = "burr", sh1 = sh1, sh2 = sh2, sc1 = sc1, sc2 = sc2, tau = tau))
  }  
}


# simulation value (Table 2) --------------------------------------------------------------

#exponential and Weibull distirbution (Table 2)
simu_p <- function(copula = "clayton") {
  if (copula == "clayton"){
    sim <- simu.Clayton(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gumbel"){
    sim <- simu.Gumbel(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "frank"){
    sim <- simu.Frank(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "fgm"){
    sim <- simu.FGM(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gb"){
    sim <- simu.GB(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  
  sim1 <- pmin(sim[,3], tau)
  sim2 <- pmin(sim[,4], tau)
  p <- NULL
  p[1] <- mean(sim[,3]>sim[,4])+1/2*mean(sim[,3]==sim[,4])
  p[2] <- mean(sim1>sim2)+1/2*mean(sim1==sim2)
  
  result_sim <- matrix(c(copula, copula_param, tau, round(p[1], 3), round(p[2], 3)), ncol = 5)
  colnames(result_sim) <- c("copula", "copula parameter", "tau", "p", "p_tau")
  return(result_sim)
}

#follow-up time
c_tau <- c(0.5, 2, 5)

#exponential
sh1 <- 1.0
sc1 <- 1.0
sh2 <- 1.0
sc2 <- 2.0

#Clayton
c_param <- c(1,5,10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "clayton"))
  }  
}

#Gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "gumbel"))
  }  
}

#Frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "frank"))
  }  
}

#FGM
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "fgm"))
  }  
}

#GB
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "gb"))
  }  
}

#Weibull
sh1 <- 0.5
sc1 <- 1.0
sh2 <- 1.0
sc2 <- 2.0

#Clayton
c_param <- c(1,5,10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "clayton"))
  }  
}

#Gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "gumbel"))
  }  
}

#Frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "frank"))
  }  
}

#FGM
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "fgm"))
  }  
}

#GB
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_p(copula = "gb"))
  }  
}

#gamma distirbution
simu_pg <- function(copula = "clayton") {
  if (copula == "clayton"){
    sim <- simu.Clayton(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gumbel"){
    sim <- simu.Gumbel(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "frank"){
    sim <- simu.Frank(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "fgm"){
    sim <- simu.FGM(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gb"){
    sim <- simu.GB(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  
  sim1 <- pmin(qgamma(1-sim[,1], shape = sh1, scale = 1/sc1), tau)
  sim2 <- pmin(qgamma(1-sim[,2], shape = sh2, scale = 1/sc2), tau)
  p <- NULL
  p[1] <- mean(qgamma(1-sim[,1], shape = sh1, scale = 1/sc1)>qgamma(1-sim[,2], shape = sh2, scale = 1/sc2))+1/2*mean(qgamma(1-sim[,1], shape = sh1, scale = 1/sc1)==qgamma(1-sim[,2], shape = sh2, scale = 1/sc2))
  p[2] <- mean(sim1>sim2)+1/2*mean(sim1==sim2)

  result_sim <- matrix(c(copula, copula_param, tau, round(p[1], 3), round(p[2], 3)), ncol = 5)
  colnames(result_sim) <- c("copula", "copula parameter", "tau", "p", "p_tau")
  return(result_sim)
}

#follow-up time
c_tau <- c(0.5, 2, 5)

#parameters
sh1 <- 1.5
sc1 <- 1.0
sh2 <- 2.0
sc2 <- 2.0

#Clayton
c_param <- c(1,5,10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pg(copula = "clayton"))
  }  
}

#Gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pg(copula = "gumbel"))
  }  
}

#Frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pg(copula = "frank"))
  }  
}

#FGM
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pg(copula = "fgm"))
  }  
}

#GB
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pg(copula = "gb"))
  }  
}


#log-normal distirbution
simu_pl <- function(copula = "clayton") {
  if (copula == "clayton"){
    sim <- simu.Clayton(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gumbel"){
    sim <- simu.Gumbel(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "frank"){
    sim <- simu.Frank(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "fgm"){
    sim <- simu.FGM(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gb"){
    sim <- simu.GB(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  
  sim1 <- qlnorm(1-sim[,1], meanlog = sh1, sdlog = sc1)
  sim2 <- qlnorm(1-sim[,2], meanlog = sh2, sdlog = sc2)
  sim1_t <- pmin(sim1, tau)
  sim2_t <- pmin(sim2, tau)
  p <- NULL
  p[1] <- mean(sim1>sim2)+1/2*mean(sim1==sim2)
  p[2] <- mean(sim1_t>sim2_t)+1/2*mean(sim1_t==sim2_t)
  
  result_sim <- matrix(c(copula, copula_param, tau, round(p[1], 3), round(p[2], 3)), ncol = 5)
  colnames(result_sim) <- c("copula", "copula parameter", "tau", "p", "p_tau")
  return(result_sim)
}

#follow-up time
c_tau <- c(0.5, 2, 5)

#parameters
sh1 <- 0.7
sc1 <- 1.5
sh2 <- 0.3
sc2 <- 2.0

#Clayton
c_param <- c(1,5,10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pl(copula = "clayton"))
  }  
}

#Gumbel
c_param <- c(0,4)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pl(copula = "gumbel"))
  }  
}

#Frank
c_param <- c(-5,1,5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pl(copula = "frank"))
  }  
}

#FGM
c_param <- c(-1,0,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pl(copula = "fgm"))
  }  
}

#GB
c_param <- c(0.5,1)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pl(copula = "gb"))
  }  
}

#Burr III distirbution
simu_pb <- function(copula = "clayton") {
  if (copula == "clayton"){
    sim <- simu.Clayton(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gumbel"){
    sim <- simu.Gumbel(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "frank"){
    sim <- simu.Frank(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "fgm"){
    sim <- simu.FGM(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  else if (copula == "gb"){
    sim <- simu.GB(100000, alpha = copula_param, scale1 = sc1, scale2 = sc2, shape1 = sh1, shape2 = sh2)
  }
  
  sim_v <- NULL
  sim_v <- cbind(((1-sim[,1])^(-1/sc1)-1)^(-1/sh1), ((1-sim[,2])^(-1/sc2)-1)^(-1/sh2))
  sim1 <- pmin(sim_v[,1], tau)
  sim2 <- pmin(sim_v[,2], tau)
  p <- NULL
  p[1] <- mean(sim_v[,1]>sim_v[,2])+1/2*mean(sim_v[,1]==sim_v[,2])
  p[2] <- mean(sim1>sim2)+1/2*mean(sim1==sim2)
  
  result_sim <- matrix(c(copula, copula_param, tau, round(p[1], 3), round(p[2], 3)), ncol = 5)
  colnames(result_sim) <- c("copula", "copula parameter", "tau", "p", "p_tau")
  return(result_sim)
}

#follow-up time
tau_c <- c(0.5, 2, 5)

#parameter
sh1 <- 1.5
sc1 <- 3.0
sh2 <- 1.0
sc2 <- 1.0

#Clayton
c_param <- c(1,5,10)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pb(copula = "clayton"))
  }  
}

#Gumbel
c_param <- c(0,4)
tau_c <- c(0.5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pb(copula = "gumbel"))
  }  
}

#Frank
c_param <- c(-5,1,5)
tau_c <- c(0.5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pb(copula = "frank"))
  }  
}

#FGM
c_param <- c(-1,0,1)
tau_c <- c(0.5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pb(copula = "fgm"))
  }  
}

#GB
c_param <- c(0.5,1)
tau_c <- c(0.5)
for (j in c_param) {
  copula_param <- j
  for (i in c_tau) {
    tau <- i
    print(simu_pb(copula = "gb"))
  }  
}